﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Web;
using Newtonsoft;
using DotNet.Integrations.Models;

namespace DotNet.Integrations.Sample
{
    public class SampleService : ISampleApi
    {

        private IApiSettingsFactory _settingsFactory;
        private static readonly string API_PATH_START = "/data/rest/json/v1/";
        private List<TicketIntegrationContext> _tickets = new List<TicketIntegrationContext>();
        private int _nextId = 1001;

        public SampleService(IApiSettingsFactory settingsFactory)
        {
            this._settingsFactory = settingsFactory;

            Add(new TicketIntegrationContext
            {
                TicketNumber = _nextId.ToString().Remove(0, 1),
                Title = "Spooooky Noises",
                Description = "Customer reports loud noise from back of coffee holder (pc)",
                FirstName = "Billy",
                LastName = "Baldwell",
                Email = "bbaldwell@exampl.com",
                Phone = "333-555-1212",
                Status = "Active",
                AssignedTo = "Jane Jellybean",
                Opened = "2104-01-09 16:12:01",
                Updated = "2014-01-09 20:43:46",
                Notes = new TicketIntegrationNotesContext[] { 
                    new TicketIntegrationNotesContext { PostedBy = "Jane Jellybean", PostedOn = "2014-01-09 19:18:01", Body = "I am looking into it. Sounds like it's a crappy computer." }, 
                    new TicketIntegrationNotesContext { PostedBy = "Jane Jellybean", PostedOn = "2014-01-09 20:43:46", Body = "Yep, I'm going to buy a new fan and maybe eventually put it in." } 
                }
            });
            Add(new TicketIntegrationContext
            {
                TicketNumber = _nextId.ToString().Remove(0, 1),
                Title = "Bad Battery",
                Description = "User's battery is leaking fluid and burning hands",
                FirstName = "Jeffrey",
                LastName = "Jehanovich",
                Email = "jjehanovich@exampl.com",
                Phone = "333-555-1212",
                Status = "Active",
                AssignedTo = "Patrick Parsons",
                Opened = "2104-01-10 12:11:01",
                Updated = "2014-01-11 19:23:46",
                Notes = new TicketIntegrationNotesContext[] { 
                    new TicketIntegrationNotesContext { PostedBy = "Patrick Parsons", PostedOn = "2014-01-11 19:25:01", Body = "The user's computer is an apple {sigh}. The service tag is X42EV3. Watiing for approval to dispose of this laptop and buy a real laptop." }, 
                    new TicketIntegrationNotesContext { PostedBy = "Patrick Parsons", PostedOn = "2014-01-11 22:43:46", Body = "Approved" } 
                }
            });
            Add(new TicketIntegrationContext
            {
                TicketNumber = _nextId.ToString().Remove(0, 1),
                Title = "Deepspace API questions",
                Description = "Deepspace Inc. would like a demo of the new API integration features. Probably should schedule a demo.",
                FirstName = "Rigley",
                LastName = "Reilson",
                Email = "rreilson@exampl.com",
                Phone = "333-555-1212",
                Status = "New",
                AssignedTo = "Joe Salesman",
                Opened = "2104-02-16 08:12:01",
                Updated = "2014-02-16 09:00:46",
                Notes = new TicketIntegrationNotesContext[] { 
                    new TicketIntegrationNotesContext { PostedBy = "BoldChat", PostedOn = "2014-02-16 12:05:05", Body = "NOTICE: This incident was created using Boldchat's integration." }, 
                }
            });
        }

        /// <summary>
        /// A way to add tickets to your own system could be implemented here; this could be exposed through a public method
        /// </summary>
        /// <param name="ticket"></param>
        TicketIntegrationContext Add(TicketIntegrationContext ticket)
        {
            if (ticket == null)
            {
                throw new ArgumentNullException("ticket");
            }

            ticket.TicketID = (this._nextId).ToString();
            this._tickets.Add(ticket);

            this._nextId += 1001;

            return ticket;
        }


        #region ISampleApi Members

        public TicketIntegrationContext GetTicket(string ticketId)
        {
            if (string.IsNullOrEmpty(ticketId))
            {
                throw new ArgumentNullException("ticketId");
            }

            return _tickets.First(t => string.Equals(t.TicketID, ticketId, StringComparison.OrdinalIgnoreCase));
        }

        public IEnumerable<TicketIntegrationContext> SearchTickets(string ticketId, string name, string email)
        {

            if (name != null && !string.IsNullOrEmpty(name))
            {
                return _tickets.FindAll(t => string.Equals(t.TicketID, (string.IsNullOrEmpty(ticketId) ? t.TicketID : ticketId)) &&
                                            (t.FirstName.ToLower().Contains(name.ToLower()) || t.LastName.ToLower().Contains(name.ToLower())) &&
                                            string.Equals(t.Email, (string.IsNullOrEmpty(email) ? t.Email : email)));
            }
            else
            {
                return _tickets.FindAll(t => string.Equals(t.TicketID, (string.IsNullOrEmpty(ticketId) ? t.TicketID : ticketId)) &&
                                            string.Equals(t.Email, (string.IsNullOrEmpty(email) ? t.Email : email)));
            }
        }

        public bool LinkTicket(IDictionary<string, string> parameters, AuthToken authToken)
        {
            //TODO: let errors bubble out to the caller of this method
            if (PerformBoldRequest(ApiMethod.UpdateChat, authToken, this._settingsFactory, parameters))
            {
                if (PerformBoldRequest(ApiMethod.UpdateChat, authToken, this._settingsFactory, parameters))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        public bool UnlinkTicket(IDictionary<string, string> parameters, AuthToken authToken)
        {
            //TODO: let errors bubble out to the caller of this method
            if (PerformBoldRequest(ApiMethod.UpdateChat, authToken, this._settingsFactory, parameters))
            {
                if (PerformBoldRequest(ApiMethod.RemoveChatIncident, authToken, this._settingsFactory, parameters))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        //public bool UpdateChat(System.Web.HttpRequest request)
        //{
        //    if (request == null)
        //    {
        //        throw new ArgumentNullException("request");
        //    }

        //    string result = PerformBoldRequest(SampleService.ApiMethod.UpdateChat, GetAuthToken(request), this._settingsFactory, request.Params);


        //    return true;
        //}

        //public bool AddChatIncident(System.Web.HttpRequest request)
        //{
        //    if (request == null)
        //    {
        //        throw new ArgumentNullException("request");
        //    }

        //    string result = PerformBoldRequest(SampleService.ApiMethod.AddChatIncident, GetAuthToken(request), this._settingsFactory, request.Params);


        //    return true;
        //}

        //public bool RemoveChatIncidents(System.Web.HttpRequest request)
        //{
        //    if (request == null)
        //    {
        //        throw new ArgumentNullException("request");
        //    }

        //    string result = PerformBoldRequest(SampleService.ApiMethod.RemoveChatIncident, GetAuthToken(request), this._settingsFactory, request.Params);


        //    return true;
        //}

        #endregion


        public enum ApiMethod
        {
            UpdateChat = 0,
            AddChatIncident = 1,
            RemoveChatIncident = 2
        }

        public bool PerformBoldRequest(ApiMethod apiMethod, AuthToken authToken, IApiSettingsFactory apiSettingsFactory, IDictionary<string, string> parms)
        {
            long? accountId = authToken.GetAccountId();
            long? integrationSettingId = authToken.GetIntegrationId();
            string serverSet = apiSettingsFactory.GetIntegrationSettings(accountId, integrationSettingId).GetServerSet();
            BoldApiRequest request = new BoldApiRequest(serverSet, string.Concat(API_PATH_START, apiMethod.ToString()), authToken.GetAccountId().GetValueOrDefault(0));
            ApiSecurity security = new ApiSecurity(apiSettingsFactory);
            request.AddParameter("auth", security.GetAPIKey(authToken.GetAccountId(), authToken.GetIntegrationId()));
            NameValueCollection requestParms = RemoveAuthParameters(parms);
            request.AddAllParameters(requestParms);

            return true;
        }


        private NameValueCollection RemoveAuthParameters(IDictionary<string, string> parms)
        {
            var newParams = new NameValueCollection();

            foreach (string key in parms.Keys)
            {
                if (!key.Equals(Filter.ApiSecurityFilter.AUTH_COOKIE_NAME) &&
                    !key.Equals(Filter.ApiSecurityFilter.AUTH_ID_COOKIE_NAME) &&
                    !key.Equals(Filter.ApiSecurityFilter.AUTH_PARAM_NAME) &&
                    !key.Equals(Filter.ApiSecurityFilter.SIGNATURE_PARAM_NAME) &&
                    !key.Equals(Filter.ApiSecurityFilter.SIGNED_PARAM_NAME))
                {
                    //newParams.Add(key, parms.TryGetValue(key));
                    string value;
                    parms.TryGetValue(key, out value);
                    if (value != null)
                    {
                        newParams.Add(key, value.ToString());
                    }
                }
            }

            return newParams;
        }
    }
}
