﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;

namespace DotNet.Integrations
{
    public class RestRequest
    {
        private readonly IRequestBase _requestBase;
        private string _path;
        private StringBuilder _parms = new StringBuilder();

        public RestRequest(IRequestBase requestBase, string path)
        {
            this._path = path;
            this._requestBase = requestBase;
        }

        public RestRequest(IRequestBase requestBase)
        {
            this._path = string.Empty;
            this._requestBase = requestBase;
        }

        public string GetPathAndParameters()
        {
            if (this._parms.Length > 0)
            {
                string parmSeperator = this._path.Contains("?") ? "&" : "?";
                return string.Concat(this._path, parmSeperator, this._parms.ToString());
            }
            else
            {
                return _path;
            }
        }

        public void AddParameter(string name, string value)
        {
            if (this._parms.Length > 0)
            {
                this._parms.Append("&");
            }
            this._parms.Append(HttpUtility.UrlEncode(name));
            if (!string.IsNullOrEmpty(value))
            {
                this._parms.Append("=");
                this._parms.Append(HttpUtility.UrlEncode(value));
            }
        }

        public string FetchRawResult()
        {
            string result = string.Empty;
            //httpurl url = new Uri(string.Concat(this._requestBase.GetUrl(), GetPathAndParameters()));
            System.Net.HttpWebRequest newRequest = (System.Net.HttpWebRequest)System.Net.WebRequest.Create(string.Concat(this._requestBase.GetUrl(), GetPathAndParameters()));
            AddAuthorizationHeader(newRequest);

            System.IO.Stream newStream = newRequest.GetRequestStream();
            System.IO.StreamReader isr = new System.IO.StreamReader(newStream);

            int numCharsRead;
            StringBuilder sb = new StringBuilder();

            while (isr.Peek() >= 0) {
                char[] charArray = new char[1024];
                numCharsRead = isr.Read(charArray, 0, charArray.Length);
                sb.Append(charArray, 0, numCharsRead);
            }

            result = sb.ToString();
            return result;
        }

        protected void AddAuthorizationHeader(System.Net.HttpWebRequest request)
        {
            if (!string.IsNullOrEmpty(this._requestBase.GetUserName()) && !string.IsNullOrEmpty(this._requestBase.GetPassword()))
            {
                request.Headers.Add("Authorization", GetAuthorizationHeader());
            }
        }

        private string GetAuthorizationHeader()
        {
            try
            {
                string authString = string.Concat(this._requestBase.GetUserName(), ":", this._requestBase.GetPassword());
                return string.Concat("Basic", Convert.ToBase64String(Encoding.UTF8.GetBytes(authString)));
            }
            catch (Exception ex)    //TODO: Capture unsupportedEncodingException
            {
                throw new Exception(ex.Message, ex.InnerException);
            }
        }



    }
}
