﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web;
using System.Web.Http;
using DotNet.Integrations.Models;

namespace DotNet.Integrations.SampleWebApi.Controllers
{
    public class SampleApiController : ApiController
    {

        private readonly Models.SampleApi _repository = new Models.SampleApi();

        /// <summary>
        /// could handle all routing in the get method
        /// </summary>
        /// <returns></returns>
        public HttpResponseMessage Get()
        {
            object tickets = null;
            var parms = RequestContext.RouteData.Values;

            if (parms.Count == 1)
            {
                if (parms.ContainsKey("controller"))
                {
                    parms.Clear();
                    foreach (var item in Request.GetQueryNameValuePairs().ToList())
                    {
                        if (!string.IsNullOrEmpty(item.Key))
                        {
                            parms.Add(item.Key, item.Value);
                        }
                    }
                }
            }

            if (parms.Count > 0)    //controller name is passed in as a route parameter
            {
                object ticketId = string.Empty;
                object name = string.Empty;
                object email = string.Empty;

                if (parms.ContainsKey("ticketid")) { parms.TryGetValue("ticketid", out ticketId); }
                if (parms.ContainsKey("name")) { parms.TryGetValue("name", out name); }
                if (parms.ContainsKey("email")) { parms.TryGetValue("email", out email); }

                try
                {
                    tickets = this._repository.SearchTickets(ticketId.ToString(), name.ToString(), email.ToString());
                }
                catch (Exception ex)
                {
                    var a = ex;
                }
            }
            else
            {
                tickets = this._repository.SearchTickets(null, null, null);
            }

            if (tickets == null)
            {
                return Request.CreateResponse(HttpStatusCode.OK, SampleWebApi.Models.SampleApi.BuildResponse(string.Empty, false));
            }

            return Request.CreateResponse(HttpStatusCode.OK, tickets);
        }

        [HttpGet]
        [Route("api/sampleapi/getticket")]
        [Route("api/sampleapi/getticket/{ticketid}")]
        public HttpResponseMessage SearchTicket(int ticketid)
        {
            var ticket = this._repository.GetTicket(ticketid.ToString());
            if (ticket == null)
            {
                return Request.CreateResponse(HttpStatusCode.OK, SampleWebApi.Models.SampleApi.BuildResponse(string.Empty, false));
            }

            return Request.CreateResponse(HttpStatusCode.OK, ticket);
        }

        [HttpGet]
        [Route("api/sampleapi/linkticket")]
        [Route("api/sampleapi/linkticket/{auth}/{chatid}/{ticketid}")]
        public HttpResponseMessage LinkTicket()
        {
            var parms = Request.GetQueryNameValuePairs().ToDictionary(x => x.Key.ToLower(), x => x.Value);
            if (parms.Count() >= 3)
            {
                string apiAuth;
                string auth;
                string chatId;
                string ticketId;

                parms.TryGetValue(Filter.ApiSecurityFilter.AUTH_COOKIE_NAME.ToString().ToLower(), out apiAuth); //apiauth
                parms.TryGetValue(Filter.ApiSecurityFilter.AUTH_PARAM_NAME.ToString().ToLower(), out auth);     //auth
                parms.TryGetValue("chatid", out chatId);
                parms.TryGetValue("customfields.ticketid", out ticketId);

                if (string.IsNullOrEmpty(auth) || string.IsNullOrEmpty(chatId) || string.IsNullOrEmpty(ticketId))
                {
                    return Request.CreateResponse(HttpStatusCode.OK, SampleWebApi.Models.SampleApi.BuildResponse("Required Parameters Missing : " + DateTime.Now.ToLongTimeString(), false));
                }

                var successResponse = this._repository.LinkTicket(parms, GetAuthToken(apiAuth, auth));
                return Request.CreateResponse(HttpStatusCode.OK, successResponse);
            }
            else
            {
                return Request.CreateResponse(HttpStatusCode.OK, SampleWebApi.Models.SampleApi.BuildResponse("Required Parameters Missing : " + DateTime.Now.ToLongTimeString(), false));
            }
        }

        private enum ServiceParameters
        {
            ChatId,
            SampleTicketNumber,
            ChatName,
            VisitEmailAddress
        }
        private string _sampleUrl = "/DotNet.Integrations.Website/#/search";

        [HttpPost]
        [Route("api/sampleapi/services")]
        [Route("api/sampleapi/services/{chatid}/{sampleticketnumber}/{chatname}/{visitemailaddress}")]
        public async System.Threading.Tasks.Task<HttpResponseMessage> Services()
        {

            string result = await Request.Content.ReadAsStringAsync();
            Dictionary<string, string> parms = null;
            if (!string.IsNullOrEmpty(result))
            {
                parms = result.Split(new[] { '&' }, StringSplitOptions.RemoveEmptyEntries)
                            .Select(x => x.Split('='))
                            .ToDictionary(a => a[0].ToLower(), a => a[1]);
            }

            System.Text.StringBuilder newPath = null;
            if (parms.Count() > 1)
            {
                string chatId;
                string sampleTicketNumber;
                string chatName;
                string visitEmailAddress;

                parms.TryGetValue(ServiceParameters.ChatId.ToString().ToLower(), out chatId);
                parms.TryGetValue(ServiceParameters.SampleTicketNumber.ToString().ToLower(), out sampleTicketNumber);
                parms.TryGetValue(ServiceParameters.ChatName.ToString().ToLower(), out chatName);
                parms.TryGetValue(ServiceParameters.VisitEmailAddress.ToString().ToLower(), out visitEmailAddress);

                appendUrlText(ref newPath, chatId, ServiceParameters.ChatId);
                appendUrlText(ref newPath, sampleTicketNumber, ServiceParameters.SampleTicketNumber);
                appendUrlText(ref newPath, chatName, ServiceParameters.ChatName);
                appendUrlText(ref newPath, visitEmailAddress, ServiceParameters.VisitEmailAddress);
                if (newPath == null) { newPath = new System.Text.StringBuilder(this._sampleUrl); }

                var response = Request.CreateResponse(HttpStatusCode.RedirectMethod);   //303
                response.Headers.Location = new Uri(string.Concat(Request.RequestUri.GetLeftPart(UriPartial.Authority), newPath.ToString()));
                return response;
            }
            else
            {
                return Request.CreateResponse(HttpStatusCode.OK, SampleWebApi.Models.SampleApi.BuildResponse("Required Parameters Missing : " + DateTime.Now.ToLongTimeString(), false));
            }
        }

        private void appendUrlText(ref System.Text.StringBuilder sb, object text, ServiceParameters key)
        {
            if (text != null)
            {
                if (sb == null)
                {
                    sb = new System.Text.StringBuilder(string.Concat(this._sampleUrl, "?"));
                }
                else
                {
                    sb.Append("&");
                }
                sb.Append(string.Concat(key.ToString(), "=", text.ToString()));
            }
        }

        /// <summary>
        /// Required parameters: apiauth, auth
        /// </summary>
        /// <param name="authCookie">apiauth</param>
        /// <param name="authParameter">auth</param>
        private AuthToken GetAuthToken(string apiauth, string auth)
        {
            string authParms = (apiauth != null) ? apiauth : auth;
            return new AuthToken(authParms);
        }

    }
}
