﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Web;
using DotNet.Integrations.Filter;
using DotNet.Integrations.Models;
using DotNet.Integrations.Sample;


namespace DotNet.Integrations.SampleWebApi.Models
{
    /// <summary>
    /// This class demo's the service that would be on your end; this would make calls down to the sample implementation we are demoing.
    /// </summary>
    public class SampleApi
    {

        private SampleService _service;

        public SampleApi()
        {
            //should really use dependency injection instead of creating a dependency on the two sample service class, but for demo purposes this is fine
            this._service = new SampleService(new SampleSettingsFactory());
        }

        public object GetTicket(string ticketId)
        {
            var ticket = this._service.GetTicket(ticketId);
            if (ticket == null)
            {
                return BuildResponse("No ticket matches the Ticket Id given. Please try again.", false);
            }
            else
            {
                //return new ReturnContext<TicketIntegrationContext>(true, ticket);
                return BuildResponse(ticket, true);
            }
        }

        public object SearchTickets(string ticketId, string name, string email)
        {
            var tickets = this._service.SearchTickets(ticketId, name, email);
            if (tickets == null)
            {
                return BuildResponse("No tickets matches the criteria given. Please try again.", false);
            }
            else
            {
                return BuildResponse(tickets, (tickets.Count() > 0));
            }
        }

        public object LinkTicket(IDictionary<string, string> parameters, AuthToken authToken)
        {
            try
            {
                bool isSuccessful = this._service.LinkTicket(parameters, authToken);
                return BuildResponse(string.Empty, isSuccessful);
            }
            catch (System.IO.IOException ex)
            {
                return BuildResponse(ex.Message, false);
            }
            catch (AuthSecurityException ex)
            {
                return BuildResponse(ex.Message, false);
            }
        }

        public object UnlinkTicket(IDictionary<string, string> parameters, AuthToken authToken)
        {
            try
            {
                bool isSuccessful = this._service.UnlinkTicket(parameters, authToken);
                return BuildResponse(string.Empty, isSuccessful);
            }
            catch (System.IO.IOException ex)
            {
                return BuildResponse(ex.Message, false);
            }
            catch (AuthSecurityException ex)
            {
                return BuildResponse(ex.Message, false);
            }
        }

        public static ReturnContext<t> BuildResponse<t>(t obj, bool isSuccess)
        {
            var returnObj = new ReturnContext<t>(isSuccess, obj);
            return returnObj;
        }
    }
}